@@ -44,6 +44,8 @@ from sklearn.metrics import accuracy_score, auc, average_precision_score, confus
 from numpy import interp
 from sklearn.metrics import  roc_auc_score, average_precision_score 
 from sklearn.model_selection import  StratifiedShuffleSplit
+import seaborn as sns
+import shutil
 #import src.Prj_Core.core_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
 #import src.Prj_Core.core_helper.model.scale_pos_weight__lgb_model as spw_lgb_model
 #import src.Prj_Core.core_helper.model.custom_bagging__lgb_model as cb_lgb_model
@@ -58,7 +60,8 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
                                   cv=False,cv_n_split=None, print_consola=True,   shuffle_cv=False,  shuffle_cv_test_size=None):
     start = time.time()
     
-    if url is not None:
+    if url is not None:        
+        #shutil.rmtree(url)
         hg.validar_directorio(url)      
     
     params.update({"metric":metric})
@@ -73,16 +76,29 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
     
     if (enable_cls_opt):
         print("----------- INICIO enable_cls_opt = True ---------------------")
-        #list_cls_ini = ['Z_NOTA_M_T_MENOS_1', 'Z_NOTA_C_T_MENOS_1', 'EXTRA_EDAD_T_MENOS_1']+hd.get_bool_columns(X_train)
-        list_cls_ini = cls_ini_opt + hd.get_bool_columns(X_train)
+        
+        list_cls_ini = cls_ini_opt 
+        
+        list_cls_ini = list(dict.fromkeys(list_cls_ini)) # removiendo duplicados  
+
+        list_cl_to_delete = []
+        for COL in list_cls_ini:
+     
+            if COL not in X_train:   
+                print(f" se eliminara la columna {COL} que no existe")                
+                list_cl_to_delete.append(COL)
+        
+                
+        list_cls_ini = hd.Diff(list_cls_ini, list_cl_to_delete) 
         
         list_cls = hd.Diff(X_train.columns.tolist(),list_cls_ini) 
         
-    
+        #print("list_cls: ", list_cls)
         list_sesgo = []
         last_best_value = 0
         
-        for COLUMN in list_cls:
+        for COLUMN in list_cls:            
+
             
             if max_iter_cls is not None:
             
@@ -91,7 +107,11 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
             
             list_cls_ini.append(COLUMN)
             
-            X_train_ = X_train.copy()             
+            X_train_ = X_train.copy()     
+            #print("X_train_ : ", X_train_.columns.tolist())
+            #print("COLUMN : ", COLUMN)
+            #print("list_cls_ini : ", list_cls_ini)
+            
             X_train_ = X_train_[list_cls_ini]
             
             X_test_ = X_test.copy()             
@@ -175,7 +195,7 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
         
 
     umbral_opt=0
-    kpis_opt = (0,0,0,0,0)
+    kpis_opt = (0,0,0,0,0,0,0,0)
     
     if enable_umbral_opt:
         umbral_opt = g.get_opt_threshold(y_test,y_prob_uno_test)   
@@ -247,17 +267,22 @@ def generate_cv_result(X_train=None,y_train=None,X_test=None,y_test=None,api=Non
 
 def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
   
+     
+
+    list_train_filtracion = [] 
+    list_train_subcobertura = [] 
     
-    list_train_sensitivity = []
-    list_train_specificity = []
-    
+    list_train_specificity = []    
     list_train_precision = []
     list_train_recall = []
     list_train_f1 = []
     list_train_average_precision = []
     list_train_roc_auc = []
     
+    list_valid_filtracion = [] 
+    list_valid_subcobertura = [] 
     
+    list_valid_specificity = []
     list_valid_precision = []
     list_valid_recall = []
     list_valid_f1 = []
@@ -284,13 +309,15 @@ def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
        train_average_precision = average_precision_score(y_train_cv, y_prob_uno_train_cv)      
        roc_auc_train = roc_auc_score(y_train_cv, y_prob_uno_train_cv) 
        
+       filtracion_train = 1 - precision_train
+       list_train_filtracion.append(filtracion_train)
+       subcobertura_train = 1 - recall_train
+       list_train_subcobertura.append(subcobertura_train)
+       
        cm1 = confusion_matrix(y_train_cv,y_pred_train_cv)
        #total1=sum(sum(cm1))
-       specificity_train = cm1[0,0]/(cm1[0,0]+cm1[0,1])
-       sensitivity_train = cm1[1,1]/(cm1[1,0]+cm1[1,1])
-       
-       
-       list_train_sensitivity.append(sensitivity_train)
+       specificity_train = cm1[0,0]/(cm1[0,0]+cm1[0,1])       
+    
        list_train_specificity.append(specificity_train)
         
        list_train_precision.append(precision_train)
@@ -313,17 +340,51 @@ def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
        precision_test, recall_test, f1_test, support_test = precision_recall_fscore_support(y_test_cv, y_pred_valid_cv ,average="binary",pos_label=1)
        valid_average_precision = average_precision_score(y_test_cv, y_prob_uno_test_cv)      
        roc_auc_valid = roc_auc_score(y_test_cv, y_prob_uno_test_cv) 
-        
+       
+              
+       filtracion_valid = 1 - precision_test
+       list_valid_filtracion.append(filtracion_valid)
+       subcobertura_valid = 1 - recall_test
+       list_valid_subcobertura.append(subcobertura_valid)
+       
+       cm11 = confusion_matrix(y_test_cv,y_pred_valid_cv)
+       specificity_valid = cm11[0,0]/(cm11[0,0]+cm11[0,1])
+       
+       list_valid_specificity.append(specificity_valid)
+               
        list_valid_precision.append(precision_test)
        list_valid_recall.append(recall_test)
        list_valid_f1.append(f1_test)
        list_valid_average_precision.append(valid_average_precision)
        list_valid_roc_auc.append(roc_auc_valid)
        
+          
+    mean_valid_subcobertura = statistics.mean(list_valid_subcobertura) 
+    std_valid_subcobertura = statistics.pstdev(list_valid_subcobertura)
+    min_valid_subcobertura = min(list_valid_subcobertura)           
+    max_valid_subcobertura = max(list_valid_subcobertura)      
+    mean_train_subcobertura = statistics.mean(list_train_subcobertura) 
+    std_train_subcobertura = statistics.pstdev(list_train_subcobertura)
+    min_train_subcobertura = min(list_train_subcobertura)           
+    max_train_subcobertura = max(list_train_subcobertura)      
    
-    mean_train_sensitivity = statistics.mean(list_train_sensitivity) 
+    mean_valid_filtracion = statistics.mean(list_valid_filtracion) 
+    std_valid_filtracion = statistics.pstdev(list_valid_filtracion)
+    min_valid_filtracion = min(list_valid_filtracion)           
+    max_valid_filtracion = max(list_valid_filtracion)      
+    mean_train_filtracion = statistics.mean(list_train_filtracion) 
+    std_train_filtracion = statistics.pstdev(list_train_filtracion)
+    min_train_filtracion = min(list_train_filtracion)           
+    max_train_filtracion = max(list_train_filtracion)     
+
+    mean_valid_specificity = statistics.mean(list_valid_specificity) 
+    std_valid_specificity = statistics.pstdev(list_valid_specificity)
+    min_valid_specificity = min(list_valid_specificity)           
+    max_valid_specificity = max(list_valid_specificity)      
     mean_train_specificity = statistics.mean(list_train_specificity) 
-    
+    std_train_specificity = statistics.pstdev(list_train_specificity)
+    min_train_specificity = min(list_train_specificity)           
+    max_train_specificity = max(list_train_specificity)     
 
     mean_valid_precision = statistics.mean(list_valid_precision)           
     std_valid_precision = statistics.pstdev(list_valid_precision)
@@ -376,24 +437,30 @@ def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
 
     
     result_kpis = [
-                   {"Metric":"Precision","Splits":n_splits,"Data":"Test","Mean":mean_valid_precision,"SD":std_valid_precision,"Min":min_valid_precision,"Max":max_valid_precision},
-                   {"Metric":"Precision","Splits":n_splits,"Data":"Train","Mean":mean_train_precision,"SD":std_train_precision,"Min":min_train_precision,"Max":max_train_precision},
+                   {"Métrica":"Precisión","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_precision,"SD":std_valid_precision,"Min":min_valid_precision,"Max":max_valid_precision},
+                   {"Métrica":"Precisión","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_precision,"SD":std_train_precision,"Min":min_train_precision,"Max":max_train_precision},
                    
-                   {"Metric":"Recall","Splits":n_splits,"Data":"Test","Mean":mean_valid_recall,"SD":std_valid_recall,"Min":min_valid_recall,"Max":max_valid_recall},
-                   {"Metric":"Recall","Splits":n_splits,"Data":"Train","Mean":mean_train_recall,"SD":std_train_recall,"Min":min_train_recall,"Max":max_train_recall},
+                   {"Métrica":"Sensibilidad","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_recall,"SD":std_valid_recall,"Min":min_valid_recall,"Max":max_valid_recall},
+                   {"Métrica":"Sensibilidad","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_recall,"SD":std_train_recall,"Min":min_train_recall,"Max":max_train_recall},
     
-                   {"Metric":"F1","Splits":n_splits,"Data":"Test","Mean":mean_valid_f1,"SD":std_valid_f1,"Min":min_valid_f1,"Max":max_valid_f1},
-                   {"Metric":"F1","Splits":n_splits,"Data":"Train","Mean":mean_train_f1,"SD":std_train_f1,"Min":min_train_f1,"Max":max_train_f1},
+                   {"Métrica":"Especificidad","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_specificity,"SD":std_valid_specificity,"Min":min_valid_specificity,"Max":max_valid_specificity},               
+                   {"Métrica":"Especificidad","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_specificity,"SD":std_train_specificity,"Min":min_train_specificity,"Max":max_train_specificity},               
+                                       
+                   {"Métrica":"F1","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_f1,"SD":std_valid_f1,"Min":min_valid_f1,"Max":max_valid_f1},
+                   {"Métrica":"F1","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_f1,"SD":std_train_f1,"Min":min_train_f1,"Max":max_train_f1},
     
-                   {"Metric":"Average Precision","Splits":n_splits,"Data":"Test","Mean":mean_valid_average_precision,"SD":std_valid_average_precision,"Min":min_valid_average_precision,"Max":max_valid_average_precision},
-                   {"Metric":"Average Precision","Splits":n_splits,"Data":"Train","Mean":mean_train_average_precision,"SD":std_train_average_precision,"Min":min_train_average_precision,"Max":max_train_average_precision},
+                   {"Métrica":"PR AUC","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_average_precision,"SD":std_valid_average_precision,"Min":min_valid_average_precision,"Max":max_valid_average_precision},
+                   {"Métrica":"PR AUC","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_average_precision,"SD":std_train_average_precision,"Min":min_train_average_precision,"Max":max_train_average_precision},
     
-                   {"Metric":"ROC AUC","Splits":n_splits,"Data":"Test","Mean":mean_valid_roc_auc,"SD":std_valid_roc_auc,"Min":min_valid_roc_auc,"Max":max_valid_roc_auc},
-                   {"Metric":"ROC AUC","Splits":n_splits,"Data":"Train","Mean":mean_train_roc_auc,"SD":std_train_roc_auc,"Min":min_train_roc_auc,"Max":max_train_roc_auc},               
+                   {"Métrica":"ROC AUC","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_roc_auc,"SD":std_valid_roc_auc,"Min":min_valid_roc_auc,"Max":max_valid_roc_auc},
+                   {"Métrica":"ROC AUC","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_roc_auc,"SD":std_train_roc_auc,"Min":min_train_roc_auc,"Max":max_train_roc_auc},               
+                  
+                   {"Métrica":"Filtración ","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_filtracion,"SD":std_valid_filtracion,"Min":min_valid_filtracion,"Max":max_valid_filtracion},
+                   {"Métrica":"Filtración ","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_filtracion,"SD":std_train_filtracion,"Min":min_train_filtracion,"Max":max_train_filtracion},               
+                 
+                   {"Métrica":"Subcobertura  ","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_subcobertura,"SD":std_valid_subcobertura,"Min":min_valid_subcobertura,"Max":max_valid_subcobertura},
+                   {"Métrica":"Subcobertura ","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_subcobertura,"SD":std_train_subcobertura,"Min":min_train_subcobertura,"Max":max_train_subcobertura},               
                   
-                   {"Metric":"sensitivity","Splits":n_splits,"Data":"Train","Mean":mean_train_sensitivity,"SD":0,"Min":0,"Max":0},               
-                   {"Metric":"specificity","Splits":n_splits,"Data":"Train","Mean":mean_train_specificity,"SD":0,"Min":0,"Max":0},               
-                   
                   ]
     
     df = pd.DataFrame(result_kpis)
@@ -419,7 +486,7 @@ def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix="")
        df_cv_fold = pd.DataFrame()   
         
        for path_ in list_path:
-           print(" path_ :  ", path_)
+           #print(" path_ :  ", path_)
            df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
            df_cv_fold = df_cv_fold.append(df_cv_fold_)
            
@@ -434,14 +501,14 @@ def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix="")
        #plt.plot(fpr, tpr, lw=1, alpha=0.3, label='ROC fold %d (AUC = %0.2f)' % (i, roc_auc))
        i += 1  
     
-    print("df_cv_fold.shape: ", df_cv_fold.shape)
-    plt.plot([0, 1], [0, 1], linestyle='--', lw=1, color='r', label='Reference', alpha=.8)  
+    #print("df_cv_fold.shape: ", df_cv_fold.shape)
+    plt.plot([0, 1], [0, 1], linestyle='--', lw=1, color='r', label='Línea de base', alpha=.8)  
    
     mean_tpr = np.mean(tprs, axis=0)
     mean_tpr[-1] = 1.0
     mean_auc = auc(mean_fpr, mean_tpr)
     std_auc = np.std(aucs)
-    plt.plot(mean_fpr, mean_tpr, color='b',label=r'Mean ROC (AUC = %0.2f $\pm$ %0.2f)' % (mean_auc, std_auc),lw=1, alpha=.8)
+    plt.plot(mean_fpr, mean_tpr, color='b',label=r'Media ROC (AUC = %0.2f $\pm$ %0.2f)' % (mean_auc, std_auc),lw=1, alpha=.8)
    
     std_tpr = np.std(tprs, axis=0)
     tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
@@ -450,8 +517,8 @@ def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix="")
 
     plt.xlim([-0.05, 1.05])
     plt.ylim([-0.05, 1.05])
-    plt.xlabel('False Positive Rate')
-    plt.ylabel('True Positive Rate')  
+    plt.xlabel('Tasa de falsos positivos')
+    plt.ylabel('Tasa de verdaderos positivos')  
     plt.title(f"Curva ROC (n_splits:{n_splits})")
     plt.legend(loc="lower right")
     #plt.show()
@@ -464,6 +531,178 @@ def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix="")
         plt.close()
 
 
+
+
+
+def pr_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
+
+    
+    y_real = []
+    y_proba = []
+       
+    for i in range(n_splits):   
+        
+       df_cv_fold = pd.DataFrame()   
+        
+       for path_ in list_path:
+           #print(" path_ :  ", path_)
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)
+                  
+           
+       y_test_cv = df_cv_fold.y_test_cv
+       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
+       
+       precision, recall, _ = precision_recall_curve(y_test_cv, y_prob_uno_test_cv)
+             
+       
+       #plt.plot(recall, precision, lw=1, alpha=0.3,  label='PR fold %d (AUC = %0.2f)' % (i, average_precision_score(y_test_cv, y_prob_uno_test_cv)))
+       plt.plot(recall, precision, lw=1, alpha=0.3)
+       
+       #plt.legend(loc="lower right")
+       #plt.gca().get_legend().remove()
+       
+       y_real.append(y_test_cv)
+       y_proba.append(y_prob_uno_test_cv)
+       
+
+       i += 1  
+    
+    y_real = np.concatenate(y_real)
+    y_proba = np.concatenate(y_proba)    
+    
+    precision, recall, _ = precision_recall_curve(y_real, y_proba)
+    plt.plot(recall, precision, color='b',label=r'Precisión-Recall (AUC = %0.2f)' % (average_precision_score(y_real, y_proba)), lw=2, alpha=.8)
+    
+    baseline = len(y_real[y_real==1]) / len(y_real)
+    #baseline = 0.2
+    #print(baseline)
+    plt.plot([0, 1], [baseline, baseline], linestyle='--', label=r'Línea de base = %0.2f' %(baseline))
+    
+    #https://stats.stackexchange.com/questions/34611/meanscores-vs-scoreconcatenation-in-cross-validation
+
+    plt.xlim([-0.05, 1.05])
+    plt.ylim([-0.05, 1.05])
+    plt.xlabel('Recall (Sensibilidad) ')
+    plt.ylabel('Precisión')  
+    plt.title(f"Curva Precisión-Recall (n_splits:{n_splits})")
+    plt.legend(loc="upper right")
+    #plt.gca().get_legend().remove()
+    #plt.show()
+    
+    if url_out is None:
+        plt.show()
+    else:
+        hg.validar_directorio(url_out)  
+        plt.savefig(url_out+f'pr_curva_cv_{postfix}.png', bbox_inches='tight')
+        plt.close()
+
+
+
+
+
+
+def get_umbral_opt(list_path=[], n_splits=5,url_out=None,postfix="",intervalo = 0.001):
+
+    
+    y_real = []
+    y_proba = []
+       
+    #opt_list = []
+    
+    for i in range(n_splits):   
+        
+       df_cv_fold = pd.DataFrame()   
+        
+       for path_ in list_path:
+           #print(" path_ :  ", path_)
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)
+                  
+           
+       y_test_cv = df_cv_fold.y_test_cv
+       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
+       
+       
+       y_real.append(y_test_cv)
+       y_proba.append(y_prob_uno_test_cv)
+       
+       #umbral_opt = g.get_opt_threshold(y_test_cv,y_prob_uno_test_cv,intervalo=intervalo) 
+       #print(umbral_opt)
+       #opt_list.append(umbral_opt)
+       
+
+       i += 1  
+    
+    y_real = np.concatenate(y_real)
+    y_proba = np.concatenate(y_proba)    
+    
+    umbral_opt = g.get_opt_threshold(y_real,y_proba,intervalo=intervalo)  
+    
+    #return umbral_opt , opt_list
+    return umbral_opt
+
+
+
+
+def distribucion_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
+
+    
+    y_real = []
+    y_proba = []
+       
+    for i in range(n_splits):   
+        
+       df_cv_fold = pd.DataFrame()   
+        
+       for path_ in list_path:
+           print(" path_ :  ", path_)
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)
+                  
+           
+       y_test_cv = df_cv_fold.y_test_cv
+       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
+       
+
+       y_real.append(y_test_cv)
+       y_proba.append(y_prob_uno_test_cv)       
+
+       i += 1  
+       
+    
+
+    
+    y_real = np.concatenate(y_real)
+    y_proba = np.concatenate(y_proba)   
+    
+    
+    sns.distplot(y_proba , hist = False, kde = True, 
+                 color = 'darkblue', 
+                 kde_kws={'linewidth': 3},
+                 rug_kws={'color': 'black'})
+    
+
+    #plt.xlim([-0.05, 1.05])
+    #plt.ylim([-0.05, 1.05])
+
+    
+    
+    plt.title('Densidad de Kernel de la probabilidad estimada')
+    plt.xlabel('Probabilidad Estimada')
+    plt.ylabel('Densidad')
+    
+    #plt.gca().get_legend().remove()
+    #plt.show()
+    
+    if url_out is None:
+        plt.show()
+    else:
+        hg.validar_directorio(url_out)  
+        plt.savefig(url_out+f'pr_curva_cv_{postfix}.png', bbox_inches='tight')
+        plt.close()
+
+
 def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None,print_consola=True):
     if url is not None:
         hg.validar_directorio(url)     
@@ -520,8 +759,14 @@ def predecir_clasificacion_binaria(model, X=None, umbral=0.5):
 
 
 
-def get_result_df(KPIs_list):
-    df = pd.DataFrame(columns=["Macro Región","Modelo","T Train","T Train Class 1","T Test","T Test Class 1",'Precision Train', 'Recall Train', 'F1 Train','Average Precision Train','ROC AUC Train', 'Precision Test', 'Recall Test', 'F1 Test','Average Precision Test','ROC AUC Test',"umbral_opt",'Precision_opt', 'Recall_opt', 'F1_opt','Average Precision opt','ROC AUC opt'])
+def get_result_df(KPIs_list,decimal=3):
+    
+    df = pd.DataFrame(columns=["Agrupación","Modelo","Total(E)","Total(E) Pos.","Total(V)","Total(V) Pos.",
+                               'Precisión(E)', 'Sensibilidad(E)', 'Especificidad(E)','F1(E)','PR AUC(E)','ROC AUC(E)', 'Filtración(E)', 'Subcobertura(E)', 
+                               'Precisión(V)', 'Sensibilidad(V)', 'Especificidad(V)','F1(V)','PR AUC(V)','ROC AUC(V)', 'Filtración(V)', 'Subcobertura(V)',
+                               "umbral_opt",
+                               'Precisión(Opt)','Sensibilidad(Opt)','Especificidad(Opt)','F1(Opt)','PR AUC(Opt)','ROC AUC(Opt)', 'Filtración(Opt)', 'Subcobertura(Opt)'])
+    
     for idx, result  in enumerate(KPIs_list):
         mr = result[0]
         
@@ -541,7 +786,7 @@ def get_result_df(KPIs_list):
         umbral_opt = result[8]
         Ks_opt = result[9]
         
-        result_rd = [mr,mod,t_train,t_train_class_1,t_test,t_test_class_1]+[round(num, 3) for num in list(Ks_train)]+[round(num, 3) for num in list(Ks)]+[umbral_opt]+[round(num_opt, 3) for num_opt in list(Ks_opt)]
+        result_rd = [mr,mod,t_train,t_train_class_1,t_test,t_test_class_1]+[round(num, decimal) for num in list(Ks_train)]+[round(num, decimal) for num in list(Ks)]+[umbral_opt]+[round(num_opt, decimal) for num_opt in list(Ks_opt)]
         df.loc[idx] = result_rd
 
         #df.loc[i] = list(KPIs)
