@@ -47,6 +47,7 @@ def plot_learning_curve(estimator, title, X, y, ylim=None, cv=None,
         plt.close()
         
         
+        
 def plot_confusion_matrix(cm, class_labels,url_dir=""):
     """Pretty prints a confusion matrix as a figure
 
@@ -88,6 +89,16 @@ def print_kpis_rendimiento_modelo(y_test,y_prob_uno,umbral,dir_name,print_consol
     average_precision = average_precision_score(y_test, y_prob_uno,pos_label=1)
     roc_auc = roc_auc_score(y_test, y_prob_uno) 
     
+    filtracion = 1 - precision
+ 
+    subcobertura = 1 - recall
+
+    
+    cm1 = confusion_matrix(y_test,y_pred)
+    #total1=sum(sum(cm1))
+    specificity = cm1[0,0]/(cm1[0,0]+cm1[0,1])      
+    
+    
     if (print_consola):
         
         print(classification_report(y_test, y_pred))
@@ -118,7 +129,7 @@ def print_kpis_rendimiento_modelo(y_test,y_prob_uno,umbral,dir_name,print_consol
             plt.savefig(full_dirname, bbox_inches='tight')
             plt.close()
         '''
-    return precision, recall, f1 , average_precision , roc_auc
+    return precision, recall, specificity, f1 , average_precision , roc_auc ,  filtracion, subcobertura
 
 
 
